#!/usr/bin/perl -w
# (c) Copyright 2003-2007. CodeWeavers, Inc.
package CXLog;
use strict;

# Define the module interface
use vars qw(@ISA @EXPORT);
use Exporter ();
@ISA    = "Exporter";
@EXPORT = qw(cxlog cxname0 cxwarn cxerr);

use FileHandle;

my $cx_log;
my $fh;
local *OLD_STDERR;
my $channels;

sub open($$)
{
    if (!defined $cx_log)
    {
        $cx_log=$_[0];
        $fh=FileHandle->new($cx_log,$_[1]) if ($cx_log);
        $fh->autoflush(1) if ($fh);
        return $fh;
    }
    return undef;
}

sub fdopen($)
{
    if (!defined $cx_log)
    {
        my ($fd)=@_;
        $cx_log="&$fd";
        $fh=FileHandle->new_from_fd($fd,"a");
        return $fh;
    }
    return undef;
}

sub close()
{
    if ($fh)
    {
        print $fh "Logging turned off\n";
        close($fh);
        undef $cx_log;
        undef $fh;
    }
    if (defined fileno OLD_STDERR)
    {
        CORE::open STDERR, ">&OLD_STDERR";
        CORE::close OLD_STDERR;
        my $tmp=select STDERR; $| = 1; select $tmp; # Make unbuffered
    }
}

sub set_channel_default($$)
{
    my ($channel, $default)=@_;
    $channels->{$channel}=$default if (!defined $channels->{$channel});
}

sub is_channel_on($)
{
    my ($channel)=@_;
    return ($fh and $channels->{$channel});
}

sub get_filename()
{
    return $cx_log;
}

sub is_on()
{
    return $fh;
}

sub cxlog(@)
{
    print $fh @_ if ($fh);
}

BEGIN {
    if (!defined $ENV{CX_LOG})
    {
        # Nothing to do then
        ;
    }
    elsif ($ENV{CX_LOG} eq "-")
    {
        CXLog::fdopen(2);
        print $fh "\n\n";
        print $fh "***** ",scalar localtime(),"\n";
        print $fh "Starting: '$0' '",join("' '",@ARGV),"'\n\n";
    }
    elsif (CXLog::open($ENV{CX_LOG},"a"))
    {
        print $fh "\n\n";
        print $fh "***** ",scalar localtime(),"\n";
        print $fh "Starting: '$0' '",join("' '",@ARGV),"'\n\n";
        if (defined fileno STDERR)
        {
            CORE::open OLD_STDERR, ">&STDERR";
            CORE::open STDERR, ">&=" . $fh->fileno();
            my $tmp=select STDERR; $| = 1; select $tmp; # Make unbuffered
        }
    }
    if (defined $ENV{CX_DEBUGMSG})
    {
        foreach my $chan_spec (split /,+/, $ENV{CX_DEBUGMSG})
        {
            $chan_spec =~ s/^([-+])//;
            $channels->{$chan_spec}=(($1 || "+") eq "+" ? 1 : 0);
        }
    }
}


#####
#
# Error and warning reporting interface
#
#####

sub cxname0()
{
    my $name0=$0;
    $name0 =~ s+^.*/++;
    return $name0;
}

sub cxwarn(@)
{
    print STDERR cxname0(), ":warning: ", @_;
}

sub cxerr(@)
{
    print STDERR cxname0(), ":error: ", @_;
}

return 1;
